unit sls_form;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DbClient, DB, myhashedstringlist;

type
  TForm1 = class(TForm)
    btnPopulate: TButton;
    memoLog: TMemo;
    btnPlainFind: TButton;
    btnSortAndFind: TButton;
    btnSortAndFind2: TButton;
    btnHashSort: TButton;
    cbKeepChanging: TCheckBox;
    GroupBox1: TGroupBox;
    edPopItems: TEdit;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    brnMyHash: TButton;
    btnMyHash2: TButton;
    btnPopCds: TButton;
    BtnFindCds: TButton;
    procedure btnPopulateClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnPlainFindClick(Sender: TObject);
    procedure btnSortAndFindClick(Sender: TObject);
    procedure btnSortAndFind2Click(Sender: TObject);
    procedure btnHashSortClick(Sender: TObject);
    procedure brnMyHashClick(Sender: TObject);
    procedure btnMyHash2Click(Sender: TObject);
    procedure btnPopCdsClick(Sender: TObject);
    procedure BtnFindCdsClick(Sender: TObject);
  private
    cds: TClientDataSet;
    sourceList: TStringList;
    function FindOne (sList: TStringList): Integer;
    function RepeatFind (sList: TStringList): Integer;
    function RepeatCdsFind: Integer;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  timetest, iniFiles; //, myhashedstringlist;

function NewRandomString: string;
var
  nLen, i: Integer;
begin
  nLen := random (30);
  SetLength (result, nLen);
  for i := 1 to nLen do
    Result [i] := Chr (48 + Random (60));
end;

procedure TForm1.btnPopulateClick(Sender: TObject);
var
  i, nPopItems: Integer;
  ttest: TTimeTest;
begin
  ttest:= TTimeTest.Create;
  try
    sourceList.Clear;
    nPopItems := StrToInt (edPopItems.Text);
    for i := 1 to nPopItems do
      sourceList.Add (NewRandomString);
    // final touch
    sourceList.Add ('MasteringDelphi');
    memoLog.Lines.Add ('Populate: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  sourceList := TStringList.Create;
  cds := TClientDataSet.Create (self);
  cds.FieldDefs.Add('text', ftString, 30);
  cds.CreateDataSet;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FreeAndNil(sourceList);
end;

function TForm1.RepeatFind (sList: TStringList): Integer;
var
  i: Integer;
  KeepChanging: Boolean;
begin
  memoLog.Lines.Add ('Finding: ' + sList[FindOne (sList)]);

  Result := 0;
  KeepChanging := cbKeepChanging.Checked;

  for i := 1 to 1000 do
    if sList.IndexOf(sList [random (sList.Count)]) >= 0 then
    begin
      inc (Result);
      if KeepChanging then
        sList.Add (NewRandomString);
    end;

  // do a final consistentcy test
  i := random (sList.Count);
  if sList [i] <> sList [sList.IndexOf (sList [i])] then
    ShowMessage ('error in find');
end;

procedure TForm1.btnPlainFindClick(Sender: TObject);
var
  ttest: TTimeTest;
begin
  ttest:= TTimeTest.Create;
  try
    RepeatFind (sourceList);
    memoLog.Lines.Add ('Plain Find: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TForm1.btnSortAndFindClick(Sender: TObject);
var
  ttest: TTimeTest;
  sSorted: TStringList;
begin
  ttest:= TTimeTest.Create;
  try
    sSorted := TStringList.Create;
    // copy then sort
    sSorted.AddStrings (sourceList);
    sSorted.Sorted := True;
    RepeatFind (sSorted);
    memoLog.Lines.Add ('Sort Find: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TForm1.btnSortAndFind2Click(Sender: TObject);
var
  ttest: TTimeTest;
  sSorted: TStringList;
begin
  ttest:= TTimeTest.Create;
  try
    sSorted := TStringList.Create;
    // sort and then copy
    sSorted.Sorted := True;
    sSorted.AddStrings (sourceList);
    RepeatFind (sSorted);
    memoLog.Lines.Add ('Sort Find 2: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TForm1.btnHashSortClick(Sender: TObject);
var
  ttest: TTimeTest;
  sHashed: THashedStringList;
begin
  ttest:= TTimeTest.Create;
  try
    sHashed := THashedStringList.Create;
    // copy (ignore sort, use hash)
    // sHashed.Sorted := True;
    sHashed.AddStrings (sourceList);
    RepeatFind (sHashed);
    memoLog.Lines.Add ('Hash Sort: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TForm1.brnMyHashClick(Sender: TObject);
var
  ttest: TTimeTest;
  sHashed: TMyHashedStringList; // was My
begin
  ttest:= TTimeTest.Create;
  try
    sHashed := TMyHashedStringList.Create; // was My
    // hash sort after adding
    sHashed.AddStrings (sourceList);
    sHashed.Sorted := True;
    RepeatFind (sHashed);
    memoLog.Lines.Add ('Hash Sort: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TForm1.btnMyHash2Click(Sender: TObject);
var
  ttest: TTimeTest;
  sHashed: TMyHashedStringList;// was My
begin
  ttest:= TTimeTest.Create;
  try
    sHashed := TMyHashedStringList.Create; // was My
    // hash sort before adding
    sHashed.Sorted := True;
    sHashed.AddStrings (sourceList);
    RepeatFind (sHashed);
    memoLog.Lines.Add ('Hash Sort: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

function TForm1.FindOne(sList: TStringList): Integer;
begin
  Result := sList.IndexOf ('MasteringDelphi');
end;

procedure TForm1.btnPopCdsClick(Sender: TObject);
var
  i, nPopItems: Integer;
  ttest: TTimeTest;
  str: string;
begin
  ttest:= TTimeTest.Create;
  try
    // sourceList.Clear;
    cds.EmptyDataSet;
    nPopItems := StrToInt (edPopItems.Text);
    for i := 1 to nPopItems do
    begin
      str := NewRandomString;
      cds.InsertRecord ([str]);
      // sourceList.Add (str);
    end;
    // final touch
    cds.InsertRecord (['MasteringDelphi']);
    // sourceList.Add ('MasteringDelphi');

    memoLog.Lines.Add ('Populate: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TForm1.BtnFindCdsClick(Sender: TObject);
var
  ttest: TTimeTest;
begin
  ttest:= TTimeTest.Create;
  try
    cds.AddIndex ('textindex', 'text', []);
    cds.IndexName := 'textindex';
    RepeatCdsFind;
    memoLog.Lines.Add ('Cds Find: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

function TForm1.RepeatCdsFind: Integer;
var
  i: Integer;
  // KeepChanging: Boolean;
begin
  // memoLog.Lines.Add ('Finding: ' + sList[FindOne (sList)]);

  Result := 0;
  // KeepChanging := cbKeepChanging.Checked;

  for i := 1 to 1000 do
  begin
    if cds.FindKey ([sourceList [random (sourceList.Count)]]) then
    begin
      inc (Result);
{      if KeepChanging then
        sList.Add (NewRandomString);}
    end;
  end;

  // do a final consistentcy test
{  i := random (sList.Count);
  if sList [i] <> sList [sList.IndexOf (sList [i])] then
    ShowMessage ('error in find');}
end;

initialization
  Randomize;
end.
