unit sls_form_dotnet;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DbClient, DB, System.Collections,
  System.ComponentModel;

type
  TFormSLS = class(TForm)
    btnPopulate: TButton;
    memoLog: TMemo;
    btnPlainFind: TButton;
    btnSortAndFind: TButton;
    GroupBox1: TGroupBox;
    edPopItems: TEdit;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    btnPopulateNet: TButton;
    btnPlainFindNet: TButton;
    btnSortFindNet: TButton;
    btnHashTable: TButton;
    procedure btnPopulateClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnPlainFindClick(Sender: TObject);
    procedure btnSortAndFindClick(Sender: TObject);
    procedure btnPopulateNetClick(Sender: TObject);
    procedure btnPlainFindNetClick(Sender: TObject);
    procedure btnSortFindNetClick(Sender: TObject);
    procedure btnHashTableClick(Sender: TObject);
  private
    sourceList: TStringList;
    SourceArray: ArrayList;
    function FindOne (sList: TStringList): Integer;
    function RepeatFind (sList: TStringList): Integer;
    function RepeatFindNet(sList: ArrayList; fSorted: Boolean = False): Integer;
    function FindOneNet(sList: ArrayList): Integer;
    function RepeatFindHash(sList: HashTable): Integer;
  public
    { Public declarations }
  end;

var
  FormSLS: TFormSLS;

implementation

{$R *.dfm}

uses
  timetest, iniFiles, System.Text; //, myhashedstringlist;

function NewRandomString: string;
var
  nLen, i: Integer;
  sb: StringBuilder;
begin
  nLen := random (300);
  // was: SetLength (result, nLen);
  sb := Stringbuilder.Create (nLen);
  for i := 1 to nLen do
    // was Result [i] := Chr (48 + Random (60));
    sb.Append(Chr (48 + Random (60)));
  Result := sb.ToString;  
end;

procedure TFormSLS.btnPopulateClick(Sender: TObject);
var
  i, nPopItems: Integer;
  ttest: TTimeTest;
begin
  ttest:= TTimeTest.Create;
  try
    sourceList.Clear;
    nPopItems := StrToInt (edPopItems.Text);
    for i := 1 to nPopItems do
      sourceList.Add (NewRandomString);
    // final touch
    sourceList.Add ('MasteringDelphi');
    memoLog.Lines.Add ('Populate: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TFormSLS.FormCreate(Sender: TObject);
begin
  sourceList := TStringList.Create;
  sourceArray := ArrayList.Create;
end;

procedure TFormSLS.FormDestroy(Sender: TObject);
begin
  FreeAndNil(sourceList);
end;

function TFormSLS.RepeatFind (sList: TStringList): Integer;
var
  i: Integer;
begin
  Result := 0;

  for i := 1 to 1000 do
    if sList.IndexOf(sList [random (sList.Count)]) >= 0 then
      inc (Result);

  memoLog.Lines.Add ('Result: ' + IntToStr (Result));
  // do a final consistency test
  i := random (sList.Count);
  if sList [i] <> sList [sList.IndexOf (sList [i])] then
    ShowMessage ('error in find');
end;

function TFormSLS.RepeatFindNet (sList: ArrayList; fSorted: Boolean): Integer;
var
  i: Integer;
begin
  Result := 0;

  for i := 1 to 1000 do
  begin
    if not fSorted then
    begin
      if sList.IndexOf(sList [random (sList.Count)]) >= 0 then
        inc (Result);
    end
    else
    begin
      if sList.BinarySearch (sList [random (sList.Count)]) >= 0 then
        inc (Result);
    end;
  end;
  memoLog.Lines.Add ('Result: ' + IntToStr (Result));
  // do a final consistency test
  i := random (sList.Count);
  if sList [i] <> sList [sList.IndexOf (sList [i])] then
    ShowMessage ('error in find');
end;

function TFormSLS.RepeatFindHash (sList: HashTable): Integer;
var
  i: Integer;
  akey: string;
begin
  memoLog.Lines.Add ('Finding: ' + sourceArray [sourceArray.IndexOf('MasteringDelphi')].ToString);

  Result := 0;

  for i := 1 to 1000 do
  begin
    akey := sourceArray [random (sourceArray.Count)].ToString;
    if sList.ContainsKey (akey) then
      inc (Result);
  end;
  memoLog.Lines.Add ('Result: ' + IntToStr (Result));
  // do a final consistency test
  // i := random (sList.Count);
  (*if sList [i] <> sList [sList.IndexOf (sList [i])] then
    ShowMessage ('error in find');*)
end;

procedure TFormSLS.btnPlainFindClick(Sender: TObject);
var
  ttest: TTimeTest;
begin
  ttest:= TTimeTest.Create;
  try
    RepeatFind (sourceList);
    memoLog.Lines.Add ('Plain Find: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TFormSLS.btnSortAndFindClick(Sender: TObject);
var
  ttest: TTimeTest;
  sSorted: TStringList;
begin
  ttest:= TTimeTest.Create;
  try
    sSorted := TStringList.Create;
    // copy then sort
    sSorted.AddStrings (sourceList);
    sSorted.Sorted := True;
    RepeatFind (sSorted);
    memoLog.Lines.Add ('Sort Find: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

function TFormSLS.FindOne(sList: TStringList): Integer;
begin
  Result := sList.IndexOf ('MasteringDelphi');
end;

function TFormSLS.FindOneNet(sList: ArrayList): Integer;
begin
  Result := sList.IndexOf ('MasteringDelphi');
end;

procedure TFormSLS.btnPopulateNetClick(Sender: TObject);
var
  i, nPopItems: Integer;
  ttest: TTimeTest;
begin
  ttest:= TTimeTest.Create;
  try
    sourceArray.Clear;
    nPopItems := StrToInt (edPopItems.Text);
    for i := 1 to nPopItems do
      sourceArray.Add (NewRandomString);
    // final touch
    sourceArray.Add ('MasteringDelphi');
    memoLog.Lines.Add ('Populate .NET: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TFormSLS.btnPlainFindNetClick(Sender: TObject);
var
  ttest: TTimeTest;
begin
  ttest:= TTimeTest.Create;
  try
    RepeatFindNet (sourceArray, False);
    memoLog.Lines.Add ('Plain Find .NET: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TFormSLS.btnSortFindNetClick(Sender: TObject);
var
  ttest: TTimeTest;
  sSorted: ArrayList;
begin
  ttest:= TTimeTest.Create;
  try
    sSorted := ArrayList.Create;
    // copy then sort
    sSorted.AddRange(SourceArray);
    // memoLog.Lines.Add ('AddRange: ' + ttest.Elapsed);
    sSorted.Sort;
    // memoLog.Lines.Add ('Sorted: ' + ttest.Elapsed);
    RepeatFindNet (sSorted, True); // use BinarySearch
    memoLog.Lines.Add ('Sort Find .NET: ' + ttest.Elapsed);
  finally
    ttest.Free;
  end;
end;

procedure TFormSLS.btnHashTableClick(Sender: TObject);
var
  ttest: TTimeTest;
  sHashed: HashTable;
  i, nSkipped: Integer;
begin
  nSkipped := 0;
  ttest:= TTimeTest.Create;
  try
    sHashed := HashTable.Create;
    // copy (ignore sort, use hash, skip duplicates!)
    for i := 0 to sourceArray.Count - 1 do
      if not sHashed.ContainsKey(sourceArray[i]) then
        sHashed.Add (sourceArray[i], nil)
      else
        Inc (nSkipped);
    RepeatFindHash (sHashed);
    memoLog.Lines.Add ('HashTable Sort .NET: ' + ttest.Elapsed);
    memoLog.Lines.Add ('Skipped: ' + IntToStr(nSkipped));
  finally
    ttest.Free;
  end;
end;

initialization
  Randomize;
end.
